/**
 * @file PlanetarySystem.h
 * @author Gawen ACKERMANN and Florian BURGENER
 * @brief Set of functions and structures allowing the simulation of a planetary system.
 * @version 1.0
 * @date 2021-12-19
 *
 */

#ifndef PLANETARY_SYSTEM_H
#define PLANETARY_SYSTEM_H

#include <stdbool.h>
#include <stdint.h>

#include "CelestialObject.h"

extern const int32_t SCREEN_WIDTH;
extern const int32_t SCREEN_HEIGHT;

typedef struct PlanetarySystem {
    CelestialObject **objects;
    int32_t objects_length;
    int32_t reference_frame_index;
    double zoom_factor;
    double previous_interval;
    bool show_names;
} PlanetarySystem;

/**
 * @brief Creates a new PlanetarySystem.
 *
 * @return PlanetarySystem*
 */
PlanetarySystem *planetary_system_create();

/**
 * @brief Destroys a PlanetarySystem.
 *
 * @param planetary_system
 */
void planetary_system_destroy(PlanetarySystem *planetary_system);

/**
 * @brief Gets the reference frame vector.
 *
 * @param planetary_system
 * @return Vector2
 */
Vector2 planetary_system_get_reference_frame(PlanetarySystem *planetary_system);

/**
 * @brief Updates the planetary system.
 *
 * @param planetary_system
 * @param interval
 */
void planetary_system_update(PlanetarySystem *planetary_system, double interval);

/**
 * @brief Draws the planetary system.
 *
 * @param planetary_system
 */
void planetary_system_draw(PlanetarySystem *planetary_system);

/**
 * @brief Draws the names of the objects.
 *
 * @param planetary_system
 */
void planetary_system_draw_object_names(PlanetarySystem *planetary_system);

#endif
