#include "Vector2.h"

#include <math.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>

Vector2 vector2_create(double x, double y) {
    return (Vector2){.x = x, .y = y};
}

Vector2 vector2_create_zero() {
    return vector2_create(0, 0);
}

Vector2 vector2_add(Vector2 a, Vector2 b) {
    return vector2_create(a.x + b.x, a.y + b.y);
}

Vector2 vector2_substract(Vector2 a, Vector2 b) {
    return vector2_create(a.x - b.x, a.y - b.y);
}

Vector2 vector2_multiply(Vector2 v, double scalar) {
    return vector2_create(v.x * scalar, v.y * scalar);
}

double vector2_dot_product(Vector2 a, Vector2 b) {
    return a.x * b.x + a.y * b.y;
}

double vector2_norm_sqr(Vector2 v) {
    return vector2_dot_product(v, v);
}

double vector2_norm(Vector2 v) {
    return sqrt(vector2_norm_sqr(v));
}

Vector2 vector2_normalize(Vector2 v) {
    double norm = vector2_norm(v);
    return vector2_create(v.x / norm, v.y / norm);
}

Vector2 vector2_fit_canvas(Vector2 v, int32_t width, int32_t height) {
    double x = width / 2.0 * (1 + v.x);
    double y = height / 2.0 * (1 + v.y);
    return vector2_create(x, y);
}

void vector2_print(Vector2 v) {
    printf("(%lf, %lf)\n", v.x, v.y);
}
