#include "drawing.h"

#include <GL/glut.h>
#include <math.h>
#include <stdint.h>
#include <string.h>

#include "PlanetarySystem.h"
#include "Vector2.h"

const int64_t PLANETARY_SYSTEM_WIDTH = 280 * 1E9;
const double LINE_WIDTH = 4.0;
const int32_t FONT_HEIGHT = 15;

Vector2 scale_position(Vector2 position, Vector2 reference_frame, double zoom_factor) {
    Vector2 unscaled_position = vector2_substract(position, reference_frame);
    // Applies the zoom.
    unscaled_position = vector2_multiply(unscaled_position, zoom_factor);

    // Converts the position in relation to the width of the planetary system.
    Vector2 scaled_position = vector2_multiply(unscaled_position, 1.0 / PLANETARY_SYSTEM_WIDTH);
    scaled_position = vector2_fit_canvas(scaled_position, SCREEN_WIDTH, SCREEN_HEIGHT);
    return scaled_position;
}

void draw_disc(Vector2 position, int32_t radius) {
    glBegin(GL_POLYGON);

    for (int32_t i = 0; i < 360; i += 1) {
        double theta = i * 3.14159265 / 180;
        double x = position.x + radius * cos(theta);
        double y = position.y + radius * sin(theta);
        glVertex2f(x, y);
    }

    glEnd();
}

void draw_line(Vector2 p1, Vector2 p2) {
    glBegin(GL_LINES);
    glVertex2f(p1.x, p1.y);
    glVertex2f(p2.x, p2.y);
    glEnd();
}

void draw_scaled_lines(Vector2 *points, int32_t points_length, Vector2 reference_frame, double zoom_factor) {
    glLineWidth(LINE_WIDTH);

    if (zoom_factor < 1) {
        glLineWidth(LINE_WIDTH * zoom_factor);
    }

    // Draws the lines between all the points.
    for (int32_t i = 0; i < points_length - 1; i += 1) {
        Vector2 p1 = scale_position(points[i], reference_frame, zoom_factor);
        Vector2 p2 = scale_position(points[i + 1], reference_frame, zoom_factor);
        draw_line(p1, p2);
    }
}

void draw_text(char *text, Vector2 position) {
    glColor3f(1.0, 1.0, 1.0);
    glRasterPos2f(position.x, position.y + FONT_HEIGHT);

    for (int32_t i = 0; i < (int32_t)strlen(text); i += 1) {
        glutBitmapCharacter(GLUT_BITMAP_9_BY_15, text[i]);
    }
}
