/**
 * @file drawing.h
 * @author Gawen ACKERMANN and Florian BURGENER
 * @brief Set of functions for drawing shapes and other things.
 * @version 1.0
 * @date 2021-12-19
 *
 */

#ifndef DRAWING_H
#define DRAWING_H

#include <stdint.h>

#include "Vector2.h"

/**
 * @brief Scales the position to the planetary system and takes into account the referential and the zoom level.
 *
 * @param position
 * @param reference_frame
 * @param zoom_factor
 * @return Vector2
 */
Vector2 scale_position(Vector2 position, Vector2 reference_frame, double zoom_factor);

/**
 * @brief Draws a disc.
 *
 * @param position
 * @param radius
 */
void draw_disc(Vector2 position, int32_t radius);

/**
 * @brief Draws a line.
 *
 * @param p1
 * @param p2
 */
void draw_line(Vector2 p1, Vector2 p2);

/**
 * @brief Draws lines and scale them to the planetary system.
 *
 * @param points
 * @param points_length
 * @param reference_frame
 * @param zoom_factor
 */
void draw_scaled_lines(Vector2 *points, int32_t points_length, Vector2 reference_frame, double zoom_factor);

/**
 * @brief Draws text at the desired position.
 *
 * @param text
 * @param position
 */
void draw_text(char *text, Vector2 position);

#endif
